# -*-python-*-

__package__    = "pyupcoming"
__version__    = "0.1"
__author__     = "Aaron Straup Cope"
__url__        = "http://www.aaronland.info/python/pyupcoming/"
__cvsversion__ = "$Revision: 1.5 $"
__date__       = "$Date: 2006/01/28 23:43:29 $"
__copyright__  = "Copyright (c) 2006 Aaron Straup Cope. Perl Artistic License."

from ConfigParser import SafeConfigParser
import os.path

import urllib
import urllib2

from elementtree.ElementTree import XML
from pdis.xpath import compile

from types import ListType

class pyupcoming :

    def __init__ (self, cfg=None) :

        if cfg == None :
            cfg = SafeConfigParser({'api_key':'','auth_token':'','always_auth':0})
            cfg.read([os.path.expanduser('~/.upcoming/upcoming.conf')])
        
        self.__cfg = cfg

        self.__endpoint = "http://upcoming.org/services/rest/"
        
        self.__require_auth = ['auth.getToken',
                               'event.add', 'event.getWatchList',
                               'metro.search',
                               'venue.add',
                               'watchlist.add','watchlist.getList','watchlist.remove',
                               'user.getMetroList','user.getWatchList']

        self.__require_post = ['event.add',
                               'venue.add',
                               'watchlist.add','watchlist.remove']
        
    #
    
    def execute_method (self, api_method, **kwargs) :

        if len(kwargs) == 0 :
            kwargs = {}
        
        kwargs['method']  = api_method
        kwargs['api_key'] = self.__cfg.get("pyupcoming","api_key")

        #

        if not kwargs.has_key('token') :
            auth = self.__cfg.getboolean("pyupcoming","always_auth")

            if not auth and kwargs.has_key('auth') :
                del kwargs['auth']
                auth = True

            if not auth and self.__requires_auth(api_method) :
                auth = True
            
            if auth :
                kwargs['token'] = self.__cfg.get("pyupcoming","auth_token")

        #
        
        return self.__api_call(**kwargs)

    #

    def endpoint (self) :
        return self.__endpoint

    #
    
    def __api_call (self, **kwargs) :
        
        app_url = self.endpoint()
        
        user_agent = "%s/%s" % (__package__,__version__)
        headers    = {'User-Agent' : user_agent}
        
        request_type = self.__request_type(kwargs['method'])

        args = urllib.urlencode(kwargs)

        #
        
        if request_type == 'GET':
            # print "%s?%s"%(app_url,args)
            req = urllib2.Request("%s?%s"%(app_url,args),None, headers)

        elif request_type == 'POST':
            req = urllib2.Request(app_url, urllib.urlencode(kwargs), headers)

        else :
            raise pyupcomingError(405,"Method Not Allowed")
            return None
        
        #

        try :
            res = urllib2.urlopen(req)
        except urllib2.HTTPError, e:
            code = e.code
            msg  = e.msg
            raise pyupcomingError(code, msg)
        except urllib2.URLError, e:
            code = 999
            msg  = e.reason
            raise pyupcomingError(code, msg)
        
        #
        
        return self.__parse_response(res)

    #

    def __request_type (self, method) :
        if method in self.__require_post :
            return "POST"
        
        return "GET"

    #
    
    def __requires_auth (self, method) :
        if method in self.__require_auth :
            return True
        
        return False
    
    #

    def __parse_response (self, res) :
        xml = res.read()
        return self.__parse_xml(xml)

    #
    
    def __parse_xml (self, xml) :
        return pyupcomingResult(xml)

#
#
#

class pyupcomingResult :

    def __init__ (self, xml) :
        self.__raw  = xml
        self.__tree = XML(xml)

    #
    
    def findnodes(self, query) :
        xpath = compile(query)
        return xpath.evaluate(self.__tree) 

    # Sssshhhh

    def findvalue (self, query) :
        res = self.findnodes(query)
        if isinstance(res, ListType) and len(res) == 1 :
            return res[0]
        else :
            return res

    #
    
    def as_string (self) :
        return self.__raw

#
#
#

class pyupcomingError (Exception) :

    def __init__ (self, code, reason) :
        self.__code   = code
        self.__reason = reason

    #
    
    def __str__ (self) :
        return "(%s) %s" % (self.code(), self.reason())

    #
    
    def code (self) :
        return self.__code

    #
    
    def reason (self) :
        if self.code() == 999 :
            return self.__reason

        try :
            xp = pyupcomingResult(self.__reason)
            return xp.find("/rsp/error/@msg")
        except :
            return self.__reason

#
#
#

if __name__ == "__main__" :

    py = pyupcoming()
    xp = py.execute_method("watchlist.getList")
        
    for e in xp.find("/rsp/watchlist[@status='watch']") :
        print e.attrib['event_id']
